/*
	IRC bot - written by Raymond Vrolijk

	raymond@webslice.nl
*/

#include "global.h"


bool	conn_hub        = FALSE,
	conn_irc        = FALSE;

char	*irc_server	= "irc.stealth.net",			// IRC server
	*hub_server	= NULL,					// hub server
	*hub_arg[10],
	*irc_arg[10],
	*appl_binary;

int	sock_hub, sock_irc;

fd_set	fd_start, fd_read;


int	main(int argc, char **argv)
{
	time_t  t, lt, nt;

	struct	s_hubmsg hub_msg[] = {

		"ADD-CHAN",	hubcmd_addchan,
		"ADD-FRIEND",	hubcmd_addfriend,
		"AUTH-SUCCESS",	hubcmd_authsuccess,
		"BOTUPGD",	hubcmd_upgrade,
		"DEL-CHAN",	hubcmd_delchan,
		"JOIN-CHANS",   join_channels,
		"KICKBAN",	hubcmd_kickban,
		"LOGIN", 	hubcmd_login,
		"NEED-INVITE",	hubcmd_needinvite,
		"NEED-OPS",	hubcmd_needops,
		"NICKNAME",	hubcmd_nickname,
		"VERSION",	hub_sendversion,

		NULL,		NULL
	};

	struct	s_ircmsg irc_msg[] = {

		"INVITE",	_INVITE,
		"JOIN",		_JOIN,
		"KICK",		_KICK,
		"MODE",		_MODE,
		"NICK",		_NICK,
		"PART",		_PART,
		"PING",		_PING,
		"PRIVMSG",	_PRIVMSG,
		"QUIT",		_QUIT,
		"376",		_376,			// RPL_ENDOFMOTD
		"352",		_352,			// RPL_ENDOFNAMES
		"422",		_376,			// ERR_NOMOTD
		"433",		_433,			// ERR_NICKNAMEINUSE
		"473",		_CANTJOINCHAN,
		"474",		_CANTJOINCHAN,
		"475",		_CANTJOINCHAN,
		NULL,		NULL
	};

    	char	*buffer = (char *) malloc(4090),
		name[31], sender[256],
		*place = buffer,
		*next;

	int	len, i, sh, si;

	struct	sockaddr_in	irc_sock, hub_sock;

	irc_sock.sin_family	= AF_INET;

	if (resolve_addr(irc_server, 6667, &irc_sock) == -1) {
		printf("Couldn't resolve %s !\n", irc_server);
		return 0;
	}


	if (hub_server != NULL) {

		hub_sock.sin_family = AF_INET;

		if (resolve_addr(hub_server, 7111, &hub_sock) == -1) {
			printf("Couldn't resolve %s !\n", hub_server);
			return 0;
		}
	}

 	srand((unsigned) time(&t));

	appl_binary = argv[0];

	if (buffer == NULL) {
		printf("Unable to malloc..\n");
		return 0;
	}

	memset(buffer, 0, 4090);

	init_users();

	daemon(1,1);

	add_chan("#fltnet");

	for(;;) {

		if (hub_server != NULL && conn_hub == FALSE) {
			sh = socket(AF_INET, SOCK_STREAM, 0);
			sock_hub = sh;

			if (connect(sock_hub, (struct sockaddr *) &hub_sock, sizeof(hub_sock)) == 0) {
				conn_hub = TRUE;
			} else	sleep(10);
		}

		if (conn_irc == FALSE) {
			si = socket(AF_INET, SOCK_STREAM, 0);
			sock_irc = si;

			if (connect(sock_irc, (struct sockaddr *) &irc_sock, sizeof(irc_sock)) == 0) {
				conn_irc = TRUE;

				setnick("kernal");
   				Send(sock_irc, "user %s 0 0 :Press CTRL-ALT-DEL twice\n", botnick);

				time(&lt);
			} else	sleep(5 + (int) rand() % 10);
		}

		FD_ZERO( &fd_read );
		FD_SET( sock_hub, &fd_read );
		FD_SET( sock_irc, &fd_read );

                if( select( FD_SETSIZE, &fd_read, NULL, NULL, NULL ) < 0 ) {
                        return;
                }

                if(conn_hub == TRUE && FD_ISSET( sock_hub, &fd_read )) {

                        // read from hub server

                        if ((len = recv(sock_hub, buffer, 4086, 0)) > 0) {

				buffer[len] = 0;

				// printf("from hub (pre): %s\n", buffer);

				while(*buffer) {

					for(next = buffer; *next; next ++)
						if (*next == '\n') break;

					*next ++ = 0;

					// printf("from hub (post): %s\n", buffer);

					parse(buffer, hub_arg, 10);

	                                for (i = 0; hub_msg[i].cmd != (char *) 0; i++) {
	                                        if (!(strcasecmp(hub_msg[i].cmd, hub_arg[0]))) hub_msg[i].func();
                                        }

					buffer = next;
				}
				buffer = place;

                        }       else conn_hub = FALSE;
                }

		if(conn_irc == TRUE && FD_ISSET( sock_irc, &fd_read )) {
			// read from IRC server

			if ((len = recv(sock_irc, buffer, 4086, 0)) > 0) {

				buffer[len] = 0;
				while (*buffer) {
					buffer = IRCparse(buffer, name, sender, &len);

					for (i = 0; irc_msg[i].cmd != (char *) 0; i++)
						if (!(strcasecmp(irc_msg[i].cmd, name))) {

							irc_msg[i].func(sender, buffer);
							break;
						}

					if (!(strcasecmp(name, "ERROR"))) {
						break;
					}
					buffer += len + 2;
				}
				buffer = place;
						
			}	else conn_irc = FALSE;


			
		}

		if (conn_irc == TRUE && time(&nt) - lt > 120) {
				join_channels();
				lt = nt;

		}

	}
}


int     resolve_addr(char *addr, int defaultport, struct sockaddr_in *sockaddr)
{
        char	sock[256], *pc;
        struct  hostent *host_ent;

	strncpy(sock, addr, 256);

        if ((pc = strchr(sock, ':')) == NULL) {
                sockaddr->sin_port = htons(defaultport);
        } else {
                sockaddr->sin_port = htons(atoi(pc+1));
        	*pc = 0;
        }

        if ((sockaddr->sin_addr.s_addr = inet_addr(sock)) == -1) {

                if ((host_ent = gethostbyname(sock)) == NULL) return -1;

                bcopy(host_ent->h_addr, (char *) &sockaddr->sin_addr.s_addr, host_ent->h_length);
        }

        return 0;
}

